#include "DFont.h"
#include "SDL_Wrappers.h"
#include <string>
#include <SDL_image.h>
#include <ctime>
#include <cstdlib>
#include <algorithm> 

enum {Red, Yellow, Green, White, Black, Grey};

int const NUMBALLS = 6;
int const NUMPEGS = 2;
int const NUMHOLES = 4;

const char * SphereFiles [NUMBALLS] = {"Images/redball.png", 
				       "Images/yellowball.png",
				       "Images/greenball.png", 
				       "Images/whiteball.png",
				       "Images/blackball.png", 
				       "Images/greyball.png"};

enum {White_P, Black_P};

SDL_Surface * Balls [NUMBALLS]; 
SDL_Surface * bg;
SDL_Surface * Pegs [NUMPEGS];
SDL_Surface * Selector;
SDL_Surface * Clear;

int const NUMTRIES = 10;

int const XINC = 45;
int const YINC = 37;

int const PegStartX = 313;
int const PegStartY = 100;
int const BallStartX = 120;
int const BallStartY = 100;
int const SelStartX = 113;
int const SelStartY = 18;

int const SelIncX = 70;

int const BallPosX[NUMHOLES] = {BallStartX, BallStartX+(XINC), BallStartX+(XINC*2), BallStartX+(XINC*3)};
int const BallPosY[NUMTRIES] = {BallStartY, BallStartY+(YINC), BallStartY+(YINC*2), BallStartY+(YINC*3),
				BallStartY+(YINC*4), BallStartY+(YINC*5), BallStartY+(YINC*6), 
				BallStartY+(YINC*7), BallStartY+(YINC*8), BallStartY+(YINC*9)};
int const PegPosX[NUMHOLES] = {PegStartX, PegStartX+(XINC), PegStartX+(XINC*2), PegStartX+(XINC*3)};
int const * PegPosY = BallPosY;



int CODE[NUMHOLES];
int GUESS[NUMHOLES];

//should generate a Permutation of the possible 4 of 6.
//note 6! = 144
void generateCode () 
{
  int mixer[NUMBALLS]={0, 1, 2, 3, 4, 5};
  srand(time(0));
  int start;
  for (int i(0); i < rand()%145; ++i) 
    next_permutation(mixer, mixer+6);
  
  start=rand()%6;
  
  for (int i(0); i < NUMHOLES; ++i)
    {
      CODE[i] = mixer[start];
      start = (start + 1) % 6;
    }
}

bool checkAnswer (SDL_Surface * Scr, int row) 
{
  SDL_Rect dest;
  int numWhite(0), numBlack(0);
  for (int i(0);  i < NUMHOLES; ++i) 
    for (int j(0); j < NUMHOLES; ++j)
      if (GUESS[i] == CODE[j])
	++numWhite;


  for (int i(0); i < NUMHOLES; ++i)
    if (GUESS[i] == CODE[i]) 
      {
	--numWhite;
	++numBlack;
      }
  
  dest.y = PegPosY[row];
   for (int i(0); i < numBlack; ++i) 
     {
       dest.x = PegPosX[i];
       SDL_BlitSurface(Pegs[Black_P], NULL, Scr, &dest);
     }
   for (int i(0); i < numWhite; ++i)
     {
       dest.x = PegPosX[numBlack+i];
       SDL_BlitSurface(Pegs[White_P], NULL, Scr, &dest);
     }
  return numBlack==4;
}

//Loads the Objects into the game
void loadObjects ()
{
  SDL_Surface * temp;
 
  for (int i(0); i < NUMBALLS; ++i)
    {
      temp = IMG_Load(SphereFiles[i]);
      SDL_SetColorKey(temp, SDL_SRCCOLORKEY, GetPixel(temp, 1, 1));
      Balls[i] = SDL_DisplayFormat(temp);
    }
  //Pegs are already alpha encoded for transparency....
  Pegs[White_P] = IMG_Load("Images/whitepeg.png");
   
  Pegs[Black_P] = IMG_Load("Images/blackpeg.png");

  Selector = IMG_Load("Images/selector.png");
  SDL_SetColorKey(Selector, SDL_SRCCOLORKEY, GetPixel(Selector, 1, 1));

  temp = IMG_Load("Images/gameboard.png");
  bg = SDL_DisplayFormat(temp);

  Clear = IMG_Load("Images/cleared.png");
}

//Loads a splash screen and converts the gameboard to display format
//saves it in bg
void splash (SDL_Surface * scr) 
{
  SDL_Surface * temp;
 
  SDL_Event event;

  SDL_BlitSurface(bg, NULL, scr, NULL);  
    
  DFont fnt1("Fonts/StoneFont.png", "Fonts/StoneFont.dat");
  fnt1.PutString(scr, CTR_HORIZ, 10, "Code Break");
  
  DFont fnt2("Fonts/ArialGold.png", "Fonts/ArialGold.dat");
  fnt2.PutString(scr, CTR_HORIZ, CTR_VERT, "by: David Leimbach");

  fnt2.PutString(scr, CTR_HORIZ, 300, "Press a key to Continue...");
    
  SDL_UpdateRect(scr, 0, 0, 0, 0);
  while (event.type != SDL_KEYDOWN)
    SDL_WaitEvent(&event);
}  

//Draws the default gameboard
void drawGameboard(SDL_Surface * Scr)
{
  SDL_Rect BallPos;

  SDL_BlitSurface(bg, NULL, Scr, NULL);

  BallPos.x = 120;
  BallPos.y = 28;
  //Red Yellow Green White Black Grey
  SDL_BlitSurface(Balls[Red], NULL, Scr, &BallPos);
  BallPos.x += 70;
  SDL_BlitSurface(Balls[Yellow], NULL, Scr, &BallPos);
  BallPos.x += 70;
  SDL_BlitSurface(Balls[Green], NULL, Scr, &BallPos);
  BallPos.x += 70;
  SDL_BlitSurface(Balls[White], NULL, Scr, &BallPos);
  BallPos.x += 70;
  SDL_BlitSurface(Balls[Black], NULL, Scr, &BallPos);
  BallPos.x += 70;
  SDL_BlitSurface(Balls[Grey], NULL, Scr, &BallPos);

  SDL_UpdateRect(Scr, 0, 0, 0, 0);
}

void clearSurface (SDL_Surface * Scr) 
{
  SDL_BlitSurface(Clear, NULL, Scr, NULL);
}

void Rejoice (SDL_Surface * Scr)
{
  SDL_Event event;
  DFont fnt("Fonts/Font.png", "Fonts/Font.dat");
  fnt.PutString(Scr, CTR_HORIZ, 20, "Congratulations");
  fnt.PutString(Scr, CTR_HORIZ, CTR_VERT, "YOU WIN!!!");
  SDL_UpdateRect(Scr, 0, 0, 0, 0);
  while (event.type != SDL_KEYDOWN)
    SDL_WaitEvent(&event);
}

void You_lose (SDL_Surface * Scr)
{
  SDL_Event event;
  DFont fnt("Fonts/Font.png", "Fonts/Font.dat");
  fnt.PutString(Scr, CTR_HORIZ, 20, "Sorry :(");
  fnt.PutString(Scr, CTR_HORIZ, CTR_VERT, "YOU LOSE!!!");
  SDL_UpdateRect(Scr, 0, 0, 0, 0);
  
  while (event.type != SDL_KEYDOWN)
    SDL_WaitEvent(&event);
}

void giveInstructions (SDL_Surface * Scr) 
{
  SDL_Surface * clear;
  DFont font("Fonts/smneon20.png", "Fonts/smneon20.dat");
  SDL_Event event;
  
  //Clear the Screen
  clear = IMG_Load("Images/cleared.png");
  SDL_BlitSurface(clear, NULL, Scr, NULL);
  font.PutString(Scr, CTR_HORIZ, 1, "It's quite simple... REALLY! ;)");
  font.PutString(Scr, CTR_HORIZ, 22, 
		 "Use the arrow keys to move the golden circle to the piece you want");
  font.PutString(Scr, CTR_HORIZ, 43, 
		 "to place and press enter.  This will place the selected piece in the");
  font.PutString(Scr, CTR_HORIZ, 64,
		 "leftmost position of the current row.  When finished you");
  font.PutString(Scr, CTR_HORIZ, 85, 
		 "the code you entered will be compared agains the generated code");
  font.PutString(Scr, CTR_HORIZ, 106,
		 "You will then <hopefully> see");
  font.PutString(Scr, CTR_HORIZ, 127,
		 "a combination of white and green pieces on the");
  font.PutString(Scr, CTR_HORIZ, 148,
		 "right side of the game board.  White pieces mean you have a correct");
  font.PutString(Scr, CTR_HORIZ, 169,
		 "piece which is not in the correct posisition.  If you have a piece");
  font.PutString(Scr, CTR_HORIZ, 190,
		 "in the correct position you will get a green piece.  The object");
  font.PutString(Scr, CTR_HORIZ, 211,
		 "is to get 4 green pieces which means you have cracked the code.");
  font.PutString(Scr, CTR_HORIZ, 459,
		 "NOW GET OUT THERE AND DO IT!!!! (Press a key)");
  SDL_UpdateRect(Scr, 0, 0, 0, 0);
  while (event.type != SDL_KEYDOWN)
    SDL_WaitEvent(&event);
}
  

void mainLoop (SDL_Surface * Scr) 
{
  SDL_Event event;
  SDL_Rect select;
  SDL_Rect place;
  SDL_Surface * temp;
  SDL_Surface * save;  //for redrawing under the selector
  bool NotPicked[NUMBALLS] = {false, false, false, false, false, false};
  bool Broken(false);
  int curSelection;
  int curRow = 0;
  int curCol = 0;

  save = SDL_CreateRGBSurface(SDL_SWSURFACE, Selector->w, Selector->h, 16, 0, 0, 0, 0);

  DFont Font("Fonts/smneon20.png", "Fonts/smneon20.dat");
  
  Font.PutString(Scr, CTR_HORIZ, 429, "Welcome!");
  Font.PutString(Scr, CTR_HORIZ, 459, "Need Instructions?(y/n)");
  select.x = SelStartX;
  select.y = SelStartY;

  SDL_UpdateRect(Scr, 0, 0, 0, 0);
  
  //See if user wants instructions
  do 
    {
      SDL_WaitEvent(&event);
	if (event.key.keysym.sym == SDLK_y)
	  giveInstructions(Scr);
    } while (event.key.keysym.sym != SDLK_y && event.key.keysym.sym  != SDLK_n);
   
  drawGameboard(Scr);
  select.w = Selector->w;
  select.h = Selector->h;
  SDL_BlitSurface(Scr, &select, save, NULL);  //save the background...
  SDL_BlitSurface(Selector, NULL, Scr, &select);
  SDL_UpdateRect(Scr, 0, 0, 0, 0);

  //Main loop starts here
  generateCode();

  curSelection = 0;
  //ignore keyup events....
  SDL_EventState(SDL_KEYUP, SDL_IGNORE);
  
  while (!Broken && curRow != 10) 
    {
      SDL_WaitEvent(&event);
      
      if (event.key.keysym.sym == SDLK_ESCAPE)
  {
    exit(1);
   }
      
      if (event.key.keysym.sym == SDLK_RIGHT)
	{
	  do {
	  curSelection = (curSelection + 1)%NUMBALLS;
	  }while (NotPicked[curSelection]);
	}
	
      else if (event.key.keysym.sym == SDLK_LEFT)
	{
	  do {
	  if (curSelection - 1 >= 0)
	    --curSelection;
	  else 
	    curSelection = NUMBALLS - 1;
	  } while(NotPicked[curSelection]);
	}
      else if (event.key.keysym.sym == SDLK_RETURN)
	{
	  NotPicked[curSelection] = true;
	  GUESS[curCol] = curSelection;
	  
	  place.x = BallPosX[curCol++];
	  place.y = BallPosY[curRow];
	  SDL_BlitSurface(Balls[curSelection], NULL, Scr, &place); 
	  SDL_UpdateRect(Scr, 0, 0, 0, 0);
	  if (curCol == 4) 
	    {
	      Broken = checkAnswer(Scr, curRow);    
	      curCol = 0;
	      ++curRow;
	      for (int i(0);  i < NUMBALLS; ++i )
		NotPicked[i] = false;
	    }
	  do {
	    curSelection = (curSelection + 1)%NUMBALLS;
	  }while (NotPicked[curSelection]);

	}
      
      SDL_BlitSurface(save, NULL, Scr, &select);
      select.x = SelStartX + (curSelection * SelIncX);

      SDL_BlitSurface(Scr, &select, save, NULL);
      SDL_BlitSurface(Selector, NULL, Scr, &select);
      SDL_UpdateRect(Scr, 0, 0, 0, 0);
    }
  if (Broken) 
    Rejoice(Scr);
  else
    You_lose(Scr);
}


int main (int argc, char * * argv) 
{
  SDL_Surface * screen;
    SDL_Event ev;
      bool playAgain(true);
  
  Initialize (SDL_INIT_VIDEO);
  SDL_WM_SetCaption("Code Break", NULL);

 
  if (argc > 1) 
    screen = SetVideoMode(640, 480, 16, SDL_SWSURFACE | SDL_FULLSCREEN);
  else
    screen = SetVideoMode(640, 480, 16, SDL_SWSURFACE);

  SDL_Event event;
  DFont Font("Fonts/Font.png", "Fonts/Font.dat");
  
  SDL_ShowCursor(0);  
  loadObjects();
  splash(screen);
  while (playAgain)
   {
           
    drawGameboard(screen);
     mainLoop(screen);
     clearSurface(screen);
     Font.PutString(screen, CTR_HORIZ, CTR_VERT, "Play Again(Y/N)");	  
     SDL_UpdateRect(screen, 0, 0, 0, 0);
     do {
        SDL_WaitEvent(&event);
	if (event.key.keysym.sym == SDLK_n)
	  playAgain = false;
	     exit(1);
      } while (event.key.keysym.sym != SDLK_y && event.key.keysym.sym  != SDLK_n);

   }
  return 0;
}



